#!/usr/bin/python

import sys

from edge import eventlog
from edge.com import db

_ADMIN_JNODE = {
    "section": "users",
    "key": "admin",
    "value": {
        "system": {
            "salt": ["apr1", "PhC4N5zl"],
            "hash": "EJ4x.P4cvmQQJ18lO4vKq.",
            "role": ["user", "admin"]
        },
        "public": {
            "name": "admin",
            "rights": {
                "archives": True,
                "modules": {
                    "system": [
                        "update",
                        "zabbix"
                    ],
                    "origins": True
                },
                "detectors": True
            }
        }
    }
}

_USER_JNODE = {
    "section": "users",
    "key": "user",
    "value": {
        "system": {
            "salt": ["apr1", "h1UhkV82"],
            "hash": "H7y1jq7mV72LwnOKVqBsX1",
            "role": ["user"]
        },
        "public": {
            "name": "user"
        }
    }
}


def _add_users(query_jnodes_list):
    """
    :type query_jnodes_list:list of dict
    :raises db.Dberror:
    """
    db_args = db.DbArgs.deserialize2()

    def db_handler(db_link):
        """
        :raises db.DbError:
        """
        with db_link, db_link.cursor() as db_cursor:
            for jnode in query_jnodes_list:
                db_cursor.execute(
                    "select errcode, errmsg, result from api_value_set(%s)",
                    [db.DbJsonArg(jnode)])

                errcode, errmsg, api_reply_jnode = db_cursor.fetchone()
                if errcode != 0:
                    eventlog.log_d('fail: api_value_set (%s)' % errmsg)
                    raise db.DbError

    db.with_db(db_args, db_handler)


def _main():
    eventlog.open_log('user-ctl', 'debug')
    eventlog.log_d('try')

    # noinspection PyBroadException
    try:
        _add_users([_ADMIN_JNODE, _USER_JNODE])  # TODO: migrate to functions from pipeline helpers
        eventlog.log_d('done')

    except:
        eventlog.log_x()
        sys.exit(-1)

    finally:
        eventlog.close_log()

    sys.exit(0)


if __name__ == '__main__':
    _main()
