#!/usr/bin/python
import glob
import os
import ujson
import shutil

from edge import eventlog

from _vpw import lsvpwc
import _admin_api

_BASE_UI_PIPELINES_JNODE_DIRPATH = "/etc/edge/setupstore/admin/actions/pipeline/"


def move_files(src, dst):
    """
    Moves file or list of files from <src> to <dst>
    :type src: str | list of str
    :param dst: str
    """

    if type(src) == list:
        for src_file in src:
            shutil.move(src_file, dst)
    else:
        shutil.move(src, dst)


def _load_json_file(json_filepath):
    """
    :type json_filepath: str
    :rtype: dict | list  | None
    """
    jnode = None
    try:
        with open(json_filepath) as json_file:
            jnode = ujson.load(json_file)

    except IOError as x:
        eventlog.log_d("fail: read %s (%s)" % (json_filepath, x))

    except ValueError as x:
        eventlog.log_d("fail: deserialize %s (%s)" % (json_filepath, x))

    return jnode


def get_used_license_count():
    """
    :rtype: int
    """
    ui_jnode_filepath_set = glob.glob(_BASE_UI_PIPELINES_JNODE_DIRPATH + "*/lpr/*.json")

    used_license_count = 0
    for ui_jnode_filepath in ui_jnode_filepath_set:
        ui_jnode = _load_json_file(ui_jnode_filepath)
        if ui_jnode:
            used_license_count += ui_jnode['rw']['license-count']

        else:
            raise RuntimeError('fail: load ui-jnode')

    return used_license_count


def get_used_license_count2(license_group):
    """
    :type license_group: str | None
    :rtype: int
    """
    ui_jnode_filepath_set = glob.glob(_BASE_UI_PIPELINES_JNODE_DIRPATH + "*/lpr/*.json")

    used_license_count = 0
    for ui_jnode_filepath in ui_jnode_filepath_set:
        ui_jnode = _load_json_file(ui_jnode_filepath)
        if ui_jnode:
            if license_group == ui_jnode['rw']['license-group']:
                used_license_count += ui_jnode['rw']['license-count']
                continue

        else:
            raise RuntimeError('fail: load ui-jnode')

    return used_license_count


def get_total_license_count():
    total_license_count = 0
    for license_key in lsvpwc(by_group=True):
        total_license_count += license_key['nthreads_max']

    return total_license_count


def restart_hasp_service():
    os.system('service aksusbd restart')

    # wait until license-manager is available
    retry_count = 0
    while retry_count < 4:
        try:
            _admin_api.get_hasp_license_manager_info()
            break
        except RuntimeError:
            retry_count += 1

    if retry_count >= 4:
        raise RuntimeError('fail: HASP license-manager unavailable')
