import time
import ujson

from _backup import *
from _edge_eventstore import es_try_get_creator_ctor
from _edge_grabcontrol import gc_try_get_creator_ctor
from _edge_lens import lens_try_get_creator_ctor
from _edge_lpr import lr_try_get_creator_ctor
from _edge_rawcodec import rc_try_get_creator_ctor
from _edge_mediagrabber import mg_try_get_creator_ctor
from _edge_radar import ra_try_get_creator_ctor
from _edge_renderer import or_try_get_creator_ctor
from _edge_rtspgrabber import rg_try_get_creator_ctor
from _edge_rtspserver import rs_try_get_creator_ctor
from _edge_speedcontrol import sc_try_get_creator_ctor
from _edge_tracker import tr_try_get_creator_ctor
from _edge_webserver import ws_try_get_creator_ctor
from _edge_mediacodecgst import mc_gst_try_get_creator_ctor
from _edge_blobstore import bs_try_get_creator_ctor
from _edge_imagemapper import im_try_get_creator_ctor
from _edge_trafficlight import tl_try_get_creator_ctor
from _file_lock import *

from edge.com import fs
import edge.pipeline_helpers as pipeline_helpers


def _locked_generate(entry_ui_pid, entry_ui_module, used_ui_modules, used_edge_modules,
                     edit_ui_jnode_handler, creator_handler):
    """
    :type entry_ui_pid: str
    :type entry_ui_module: UiIdentity
    :type used_edge_modules: set of EdgeIdentity
    :type used_ui_modules: set of UiIdentity
    :type edit_ui_jnode_handler: None | (dict of UiIdentity: dict) -> bool
    :type creator_handler: (dict of UiIdentity: dict) -> None
    """

    # define entry-ui-jnode
    ui_identity_to_jnode_map = dict()
    for ui_identity in used_ui_modules:
        ui_jnode_filepath = get_ui_pipeline_jnode_filepath(entry_ui_pid, ui_identity.type, ui_identity.id)
        ui_identity_to_jnode_map[ui_identity] = fs.load_json2(ui_jnode_filepath)

    if edit_ui_jnode_handler:
        any_jnode_changed = edit_ui_jnode_handler(ui_identity_to_jnode_map)
        if any_jnode_changed is None:
            eventlog.log_d('fail: jnode_handler should return bool')
            raise RuntimeError

        elif not any_jnode_changed:
            return

    entry_ui_jnode = ui_identity_to_jnode_map.get(entry_ui_module)

    # create backup directory.
    action = '%d-pipeline-%s-%s-%s' % (int(round(time.time() * 1e6)), entry_ui_pid,
                                       entry_ui_module.type, entry_ui_module.id)

    backup_dir_path = fs.join_path(BASE_BACKUP_DIRPATH, action)
    try:
        if not os.path.exists(backup_dir_path):
            os.makedirs(backup_dir_path)

    except os.error as x:
        eventlog.log_d("fail: mkdir %s (%s)" % (backup_dir_path, x.message))
        raise RuntimeError

    backup_error = False

    # backup edge_jnode
    dir_backup_list = []
    for edge_idnetity in used_edge_modules:
        dir_backup = OldDirBackup.try_create(backup_dir_path, entry_ui_pid, edge_idnetity)
        if dir_backup is not None:
            dir_backup_list.append(dir_backup)
        else:
            eventlog.log_d('fail: create backup')
            backup_error = True

    # backup ui_jnode
    file_backup_list = []
    for ui_identity in used_ui_modules:
        file_backup = OldFileBackup.try_create(backup_dir_path, entry_ui_pid, ui_identity)
        if file_backup is not None:
            file_backup_list.append(file_backup)
        else:
            eventlog.log_d('fail: create backup')
            backup_error = True

    if entry_ui_jnode:
        # entry_ui_jnode is None in case of entry_point == PIPELINE
        new_ui_backup = NewFileBackup.try_create(backup_dir_path, entry_ui_pid, entry_ui_module, entry_ui_jnode)
        if new_ui_backup is not None:
            file_backup_list.append(new_ui_backup)

    if backup_error:
        raise RuntimeError('fail: create backup')

    # run configuration creators
    backup_error = True
    # noinspection PyBroadException
    try:
        creator_handler(ui_identity_to_jnode_map)

        for ui_identity, ui_jnode in ui_identity_to_jnode_map.iteritems():
            ui_jnode_filepath = get_ui_pipeline_jnode_filepath(entry_ui_pid, ui_identity.type, ui_identity.id)
            with open(ui_jnode_filepath, 'w') as json_file:
                ujson.dump(ui_jnode, json_file, indent=2)

        # noinspection PyBroadException
        try:
            for dir_backup in dir_backup_list:
                dir_backup.remove()

            for file_backup in file_backup_list:
                file_backup.remove()

            if not os.listdir(backup_dir_path):
                os.rmdir(backup_dir_path)
        except Exception:
            eventlog.log_x()

        backup_error = False
    except Exception:
        eventlog.log_x()

    # restore backup
    if backup_error:  # TODO: what if except inside?
        for file_backup in file_backup_list:
            file_backup.try_restore()

        for dir_backup in dir_backup_list:
            dir_backup.try_restore()

        raise RuntimeError('fail: generate')


def _generate2(entry_ui_pid, entry_ui_module, used_ui_modules, used_edge_modules,
               edit_ui_jnode_handler, creator_handler):
    """
    :type entry_ui_pid: str
    :type entry_ui_module: UiIdentity
    :type used_edge_modules: set of EdgeIdentity
    :type used_ui_modules: set of UiIdentity
    :type edit_ui_jnode_handler: None | (dict of UiIdentity: dict) -> bool
    :type creator_handler: (dict of UiIdentity: dict) -> None
    """
    lock = LockUiJnodeSet(entry_ui_pid, used_ui_modules)

    try:
        _locked_generate(
            entry_ui_pid, entry_ui_module,
            used_ui_modules, used_edge_modules,
            edit_ui_jnode_handler, creator_handler)
    finally:
        lock.release()


def generate(ui_pid, entry_point, edit_ui_jnode_handler, avaliable_ui_modules=None):
    """
    :type ui_pid: str
    :type entry_point: UiIdentity
    :type edit_ui_jnode_handler: None | (dict of UiIdentity: dict) -> bool
    :type avaliable_ui_modules: None  | set of UiIdentity
    :raises: RuntimeError
    """
    used_ui_modules = set()
    used_edge_modules = set()

    if avaliable_ui_modules is None:
        with open(pipeline_helpers.PIPELINE_SET_FILEPATH, 'r') as json_file:
            pipeline_set_jnode = ujson.load(json_file)

        pipeline_jnode = filter(lambda pipeline: pipeline['id'] == ui_pid, pipeline_set_jnode)
        avaliable_ui_modules = set()
        if len(pipeline_jnode) == 1:
            ptype = pipeline_jnode[0]['type']
            # avaliable_ui_modules.add(UiIdentity(ptype, '', ''))
            for module_jnode in pipeline_jnode[0]['modules']:
                avaliable_ui_modules.add(UiIdentity(ptype, module_jnode['type'], module_jnode['id']))

            for detector_jnode in pipeline_jnode[0]['detectors']:
                avaliable_ui_modules.add(UiIdentity(ptype, detector_jnode['type'], detector_jnode['id']))

    creator_ctor_list = filter(
        bool, map(lambda ctor: ctor(entry_point, used_ui_modules, used_edge_modules, avaliable_ui_modules),
                  [or_try_get_creator_ctor, rg_try_get_creator_ctor, lr_try_get_creator_ctor,
                   tr_try_get_creator_ctor, mc_gst_try_get_creator_ctor, bs_try_get_creator_ctor,
                   sc_try_get_creator_ctor, es_try_get_creator_ctor, ws_try_get_creator_ctor,
                   mg_try_get_creator_ctor, ra_try_get_creator_ctor, lens_try_get_creator_ctor,
                   rs_try_get_creator_ctor, gc_try_get_creator_ctor, rc_try_get_creator_ctor,
                   im_try_get_creator_ctor, tl_try_get_creator_ctor]))

    _generate2(
        ui_pid, entry_point, used_ui_modules, used_edge_modules, edit_ui_jnode_handler,
        lambda creator_args:
        map(lambda creator: creator.serialize(),
            map(lambda ctor: ctor(ui_pid, entry_point, creator_args), creator_ctor_list)))

    edge_group = helpers.get_edge_group(ui_pid)

    for edge_module in used_edge_modules:
        edge_id = helpers.get_edge_id(edge_group, edge_module)
        if edge_module.type == EDGE_TYPE_WEBSERVER:
            helpers.restart_webserver_lazy()
        elif edge_module.type == EDGE_TYPE_RADAR:
            pass
        elif edge_module.type == EDGE_TYPE_EVENTSTORE:
            pass
        elif edge_module.type == EDGE_TYPE_IMAGE_MAPPER:
            pass
        elif edge_module.type == EDGE_TYPE_TRAFFIC_LIGHT:
            helpers.sighup_setupstore()
        else:
            helpers.restart_edge_module(edge_module.type, edge_id)
