from edge import eventlog
from edge.image.types import Blob

from edge.com import bs

from .types import BsServiceArgs
from ..model.types import Identity


class IBsService:
    def __init__(self):
        pass

    def save(self, sender, blob):
        """
        :type sender: Identity
        :type blob: Blob
        :raise bs.BsError:
        """
        raise NotImplementedError


def init_service(args, counters):
    """
    :type counters: bs.IBsCounters
    :type args: BsServiceArgs
    :rtype: IBsService
    """
    return _BsService(args, counters)


def fini_service(service):
    """
    :type service: IBsService | _BsService
    """
    service.destroy()


class _BsService(IBsService):
    def __init__(self, args, counters):
        """
        :type counters: bs.IBsCounters
        :type args: BsServiceArgs
        """
        IBsService.__init__(self)
        self._session_args = counters, 65535, bs.LOG_LEVEL_NAME_TO_CODE_MAP.get(args.log_level, 'info')
        self._session = bs.Session(counters=counters, closed=True)

        self._counters = counters
        self._sender_to_chan_map = args.sender_to_chan_map

    def destroy(self):
        self._session.close()

    def save(self, sender, blob):
        """
        :type sender: Identity
        :type blob: Blob
        """
        chan_or_profiles = self._sender_to_chan_map[sender]
        if dict == type(chan_or_profiles):
            chan = chan_or_profiles[blob.profile]
        else:
            chan = chan_or_profiles

        self._counters.inc_blob(chan)
        self.save2(chan, blob)

    @bs.reopen_session
    def save2(self, chan, blob):
        """
        :type chan: str
        :type blob: Blob

        :raise bs.BsError:
        :raise bs.BsSessionError:
        """
        try:
            if blob.file_path is None:
                ident = self._session.save_blob(chan, blob.min_ts / 1000, blob.file_data, blob.file_meta)
            else:
                ident = self._session.save_blob2(chan, blob.min_ts / 1000, blob.file_path, blob.file_meta)

            blob.name = '%s_%s.%s' % (chan, ident, blob.file_ext)

        except bs.BsError:
            eventlog.log_d('fail: save <chan=%s, profile=%s>' % (str(chan), blob.profile))
            raise
