import ujson

from edge import eventlog
from edge.queue import JsonFileQueue
from edge.image.types import MemBlob
from edge.service.tz import ITzService
from edge.com.bs import BsError

from .stats import EventCounters
from .types import EventServiceArgs
from ..model.types import Event
from ..services.bs import IBsService
from ..services.systest import ISystestService


class IEventService:
    def __init__(self):
        pass

    def emit(self, event_args):
        """
        :type event_args: Event
        """
        raise NotImplementedError


def init_service(counters, args, tz_service, bs_service, systest_service):
    """
    :type counters: EventCounters
    :type args: EventServiceArgs
    :type tz_service: ITzService
    :type bs_service: IBsService
    :type systest_service: ISystestService
    :rtype: IEventService
    """
    json_q_list = map(lambda json_q_args: JsonFileQueue(json_q_args), args.json_queue_list)
    return _EventService(counters, tz_service, bs_service, systest_service, json_q_list)


# noinspection PyUnusedLocal
def fini_service(service):
    """
    :type service: IEventService | _EventService
    """
    pass


class _EventService(IEventService):
    def __init__(self, counters, tz_service, bs_service, systest_service, json_q_list):
        """
        :type counters: EventCounters
        :type tz_service: ITzService
        :type bs_service: IBsService
        :type systest_service: ISystestService
        :type json_q_list: list[JsonFileQueue]
        """
        IEventService.__init__(self)

        self._counters = counters

        self._tz_service = tz_service
        self._bs_service = bs_service
        self._systest_service = systest_service
        self._json_q_list = json_q_list

    def emit(self, event_args):
        """
        :type event_args: Event
        """
        # noinspection PyBroadException
        try:
            map(lambda blob: self._bs_service.save(event_args.sender, blob), event_args.get_blobs())

            event_jnode = event_args.serialize(self._tz_service)
            self._systest_service.serialize(event_jnode)

            event_blob = MemBlob('args', event_args.ts, ujson.dumps(event_jnode), file_ext='json')
            self._bs_service.save(event_args.sender, event_blob)

            map(lambda json_q: json_q.add_jnode(event_jnode, event_blob.name), self._json_q_list)
            self._counters.inc_fact()

        except BsError:
            eventlog.log_d('fail: save to bs skip event')

        except Exception:
            eventlog.log_x()
