from .test import Test, TestStep, Summary, TestArgsError, TestError

from edge.admin.cgi import CgiBadRequestError
from edge import eventlog
from edge import pipeline_helpers as oh

from misc import mk_vs_bbox4, get_rcs


class SpotSpeedTestStep(TestStep):
    def __init__(self, services, session_ref, test_ref, step_ref):
        """
        :type services: Services
        :type session_ref: str
        :type test_ref: (str, str)
        :type step_ref: str
        """
        TestStep.__init__(self, services, session_ref, test_ref, step_ref)

    def _edit_log_item_jnode(self, log_item_jnode, body_jnode):
        """
        :type log_item_jnode: dict
        :type body_jnode: dict
        :rtype: dict
        """

        # unpack #/data/report
        report = body_jnode.get('report', False)
        if type(report) != bool:
            eventlog.log_i('fail: http400. param #/report has invalid value')
            raise CgiBadRequestError

        try:
            data_jnode = body_jnode['data']
            speed_jnode = data_jnode['speed']
            dir_jnode = data_jnode['dir']

            # unpack #/data/speed/actual
            if speed_jnode.get('actual'):
                eventlog.log_i('fail: http400. param #/data/speed/actual is provided')
                raise CgiBadRequestError

            # unpack #/data/speed/expected
            expected_speed = speed_jnode.get('expected')
            if expected_speed is not None:
                if type(expected_speed) not in [float, int]:
                    eventlog.log_i('fail: http400. param #/data/speed/expected has invalid value')
                    raise CgiBadRequestError

            # unpack #/data/dir/actual
            if dir_jnode.get('actual'):
                eventlog.log_i('fail: http400. param #/data/dir/actual is provided')
                raise CgiBadRequestError

            # unpack #/data/dir/expected
            expected_dir = dir_jnode.get('expected')
            if expected_dir is not None:
                if type(expected_dir) not in [str, unicode]:
                    eventlog.log_i('fail: http400. param #/data/dir/expected has invalid value')
                    raise CgiBadRequestError

            # unpack #/summary
            summary_jnode = Summary.repack(body_jnode.get('summary'))

            # apply changes
            log_item_jnode['report'] = report
            log_item_jnode['data']['speed']['expected'] = expected_speed
            log_item_jnode['data']['dir']['expected'] = expected_dir
            log_item_jnode['summary'] = summary_jnode
            return log_item_jnode

        except (KeyError, ValueError) as x:
            eventlog.log_i('fail: http400 <{}>'.format(x.message))
            raise CgiBadRequestError


class SpotSpeedTest(Test):
    def __init__(self, services, session_ref, test_ref):
        """
        :type services: Services
        :type session_ref: str
        :type test_ref: (str, str)
        """
        Test.__init__(self, services, session_ref, test_ref)

    def _mk_step_jnode(self, test_jnode, step_jnode, args_jnode):
        """
        :type test_jnode: dict
        :type step_jnode: dict
        :type args_jnode: dict | None

        :raise TestArgsError:
        :raise TestError:
        """
        if args_jnode is None:
            raise TestArgsError

        from edge.com import net
        origin = test_jnode['ro']['origin']
        origin_pid = oh.get_origin_pid(origin)

        rn_bbox4 = args_jnode.get('bbox-4')
        vs_bbox4 = mk_vs_bbox4(origin_pid, rn_bbox4)
        tout = args_jnode.get('tout')
        sym_type = args_jnode.get('simulator')

        try:
            net.query_es_test(self._session_ref, self._test_ref, origin, vs_bbox4, tout)
        except net.NetError:
            eventlog.log_e('fail: setup eventstore')
            raise TestError

        # reconfig radar
        speed_filter_args = args_jnode.get('speed')
        ra_filter_speed = 0
        if speed_filter_args:
            speed_kmh, direction = speed_filter_args

            speed_mps = speed_kmh / 3.6
            speed_mps = int(round(speed_mps))
            ra_filter_speed = speed_mps if direction == 'receding' else 100 + speed_mps

        try:
            ra_origin_pid = oh.get_ra_origin_pid()
            if ra_origin_pid:
                if tout > 0:
                    min_rcs = get_rcs(origin_pid, sym_type, systest_active=True)
                    if min_rcs is None:
                        eventlog.log_e('fail: setup radar')
                        raise TestError

                    net.query_ra_alignment(ra_origin_pid, min_rcs=min_rcs, alignment=True, speed_filter=ra_filter_speed)
                else:
                    min_rcs = get_rcs(origin_pid, sym_type, systest_active=False)
                    if min_rcs is None:
                        eventlog.log_e('fail: setup radar')
                        raise TestError

                    net.query_ra_alignment(ra_origin_pid, min_rcs=min_rcs, alignment=False)

        except net.NetError:
            eventlog.log_e('fail: setup radar')
            raise TestError
